/* automatically generated from xfpm-settings.ui */
#ifdef __SUNPRO_C
#pragma align 4 (xfpm_settings_ui)
#endif
#ifdef __GNUC__
static const char xfpm_settings_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char xfpm_settings_ui[] =
#endif
{
  "<?xml version=\"1.0\"?><interface><object class=\"GtkAdjustment\" id=\""
  "adjustment1\"><property name=\"value\">14</property><property name=\"lo"
  "wer\">14</property><property name=\"upper\">360</property><property nam"
  "e=\"step_increment\">1</property><property name=\"page_increment\">10</"
  "property><property name=\"page_size\">10</property></object><object cla"
  "ss=\"GtkAdjustment\" id=\"adjustment2\"><property name=\"value\">17</pr"
  "operty><property name=\"upper\">60</property><property name=\"step_incr"
  "ement\">1</property></object><object class=\"GtkAdjustment\" id=\"adjus"
  "tment3\"><property name=\"value\">15</property><property name=\"upper\""
  ">60</property><property name=\"step_increment\">1</property><property n"
  "ame=\"page_size\">10</property></object><object class=\"GtkAdjustment\""
  " id=\"adjustment4\"><property name=\"value\">10</property><property nam"
  "e=\"lower\">9</property><property name=\"upper\">120</property><propert"
  "y name=\"step_increment\">1</property></object><object class=\"GtkAdjus"
  "tment\" id=\"adjustment5\"><property name=\"value\">15</property><prope"
  "rty name=\"lower\">14</property><property name=\"upper\">360</property>"
  "<property name=\"step_increment\">1</property><property name=\"page_inc"
  "rement\">10</property><property name=\"page_size\">10</property></objec"
  "t><object class=\"GtkAdjustment\" id=\"adjustment6\"><property name=\"v"
  "alue\">5</property><property name=\"upper\">60</property><property name"
  "=\"step_increment\">1</property></object><object class=\"GtkAdjustment\""
  " id=\"adjustment7\"><property name=\"value\">5</property><property name"
  "=\"upper\">60</property><property name=\"step_increment\">1</property><"
  "/object><object class=\"GtkAdjustment\" id=\"adjustment8\"><property na"
  "me=\"value\">10</property><property name=\"lower\">9</property><propert"
  "y name=\"upper\">120</property><property name=\"step_increment\">1</pro"
  "perty></object><object class=\"GtkAdjustment\" id=\"adjustment9\"><prop"
  "erty name=\"value\">5</property><property name=\"lower\">1</property><p"
  "roperty name=\"upper\">20</property><property name=\"step_increment\">1"
  "</property></object><object class=\"GtkListStore\" id=\"model1\"><colum"
  "ns><column type=\"gchararray\"/></columns><data><row><col id=\"0\" tran"
  "slatable=\"yes\">Nothing</col></row></data></object><object class=\"Gtk"
  "ListStore\" id=\"model3\"><columns><column type=\"gchararray\"/></colum"
  "ns><data><row><col id=\"0\" translatable=\"yes\">Nothing</col></row></d"
  "ata></object><object class=\"GtkListStore\" id=\"model4\"><columns><col"
  "umn type=\"gchararray\"/></columns><data><row><col id=\"0\" translatabl"
  "e=\"yes\">Nothing</col></row></data></object><object class=\"GtkListSto"
  "re\" id=\"model5\"><columns><column type=\"gchararray\"/></columns><dat"
  "a><row><col id=\"0\" translatable=\"yes\">Nothing</col></row></data></o"
  "bject><object class=\"GtkListStore\" id=\"model6\"><columns><column typ"
  "e=\"gchararray\"/></columns><data><row><col id=\"0\" translatable=\"yes"
  "\">Nothing</col></row></data></object><object class=\"GtkListStore\" id"
  "=\"model7\"><columns><column type=\"gchararray\"/></columns><data><row>"
  "<col id=\"0\" translatable=\"yes\">Nothing</col></row></data></object><"
  "object class=\"XfceTitledDialog\" id=\"xfpm-settings-dialog\"><property"
  " name=\"title\" translatable=\"yes\">Xfce Power Manager</property><prop"
  "erty name=\"window_position\">center-on-parent</property><property name"
  "=\"icon_name\">xfpm-ac-adapter</property><property name=\"type_hint\">d"
  "ialog</property><property name=\"subtitle\" translatable=\"yes\">Power "
  "manager settings</property><child internal-child=\"vbox\"><object class"
  "=\"GtkVBox\" id=\"dialog-vbox\"><property name=\"visible\">True</proper"
  "ty><child><object class=\"GtkVBox\" id=\"allbox\"><property name=\"visi"
  "ble\">True</property><property name=\"border_width\">6</property><child"
  "><object class=\"GtkHBox\" id=\"hbox\"><property name=\"visible\">True<"
  "/property><property name=\"spacing\">6</property><child><object class=\""
  "GtkViewport\" id=\"viewport1\"><property name=\"visible\">True</propert"
  "y><child><object class=\"GtkTreeView\" id=\"treeview\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">True</property><p"
  "roperty name=\"headers_visible\">False</property><property name=\"enabl"
  "e_search\">False</property><signal name=\"cursor_changed\" handler=\"_c"
  "ursor_changed_cb\"/></object></child></object><packing><property name=\""
  "expand\">False</property><property name=\"fill\">False</property><prope"
  "rty name=\"position\">0</property></packing></child><child><object clas"
  "s=\"GtkNotebook\" id=\"main-notebook\"><property name=\"visible\">True<"
  "/property><property name=\"can_focus\">True</property><property name=\""
  "show_tabs\">False</property><property name=\"show_border\">False</prope"
  "rty><child><object class=\"GtkViewport\" id=\"general\"><property name="
  "\"visible\">True</property><child><object class=\"GtkFrame\" id=\"frame"
  "\"><property name=\"visible\">True</property><property name=\"border_wi"
  "dth\">12</property><property name=\"label_xalign\">0</property><propert"
  "y name=\"shadow_type\">none</property><child><object class=\"GtkAlignme"
  "nt\" id=\"alignment\"><property name=\"visible\">True</property><proper"
  "ty name=\"left_padding\">12</property><property name=\"right_padding\">"
  "6</property><child><object class=\"GtkVBox\" id=\"vbox17\"><property na"
  "me=\"visible\">True</property><property name=\"orientation\">vertical</"
  "property><child><object class=\"GtkTable\" id=\"table3\"><property name"
  "=\"visible\">True</property><property name=\"n_rows\">4</property><prop"
  "erty name=\"n_columns\">2</property><property name=\"column_spacing\">1"
  "2</property><child><object class=\"GtkComboBox\" id=\"power-combox\"><p"
  "roperty name=\"visible\">True</property><property name=\"model\">model1"
  "</property><signal name=\"changed\" handler=\"button_power_changed_cb\""
  "/><child><object class=\"GtkCellRendererText\" id=\"renderer1\"/><attri"
  "butes><attribute name=\"text\">0</attribute></attributes></child></obje"
  "ct><packing><property name=\"left_attach\">1</property><property name=\""
  "right_attach\">2</property><property name=\"top_attach\">1</property><p"
  "roperty name=\"bottom_attach\">2</property><property name=\"y_padding\""
  ">3</property></packing></child><child><object class=\"GtkComboBox\" id="
  "\"sleep-combox\"><property name=\"visible\">True</property><property na"
  "me=\"model\">model4</property><signal name=\"changed\" handler=\"button"
  "_sleep_changed_cb\"/><child><object class=\"GtkCellRendererText\" id=\""
  "renderer4\"/><attributes><attribute name=\"text\">0</attribute></attrib"
  "utes></child></object><packing><property name=\"left_attach\">1</proper"
  "ty><property name=\"right_attach\">2</property><property name=\"top_att"
  "ach\">3</property><property name=\"bottom_attach\">4</property><propert"
  "y name=\"y_padding\">3</property></packing></child><child><object class"
  "=\"GtkLabel\" id=\"sleep-label\"><property name=\"visible\">True</prope"
  "rty><property name=\"xalign\">0</property><property name=\"label\" tran"
  "slatable=\"yes\">When sleep button is pressed:</property></object><pack"
  "ing><property name=\"top_attach\">3</property><property name=\"bottom_a"
  "ttach\">4</property><property name=\"x_options\">GTK_FILL</property></p"
  "acking></child><child><object class=\"GtkComboBox\" id=\"hibernate-comb"
  "ox\"><property name=\"visible\">True</property><property name=\"model\""
  ">model3</property><signal name=\"changed\" handler=\"button_hibernate_c"
  "hanged_cb\"/><child><object class=\"GtkCellRendererText\" id=\"renderer"
  "3\"/><attributes><attribute name=\"text\">0</attribute></attributes></c"
  "hild></object><packing><property name=\"left_attach\">1</property><prop"
  "erty name=\"right_attach\">2</property><property name=\"top_attach\">2<"
  "/property><property name=\"bottom_attach\">3</property><property name=\""
  "y_padding\">3</property></packing></child><child><object class=\"GtkLab"
  "el\" id=\"hibernate-label\"><property name=\"visible\">True</property><"
  "property name=\"xalign\">0</property><property name=\"label\" translata"
  "ble=\"yes\">When hibernate button is pressed:</property></object><packi"
  "ng><property name=\"top_attach\">2</property><property name=\"bottom_at"
  "tach\">3</property><property name=\"x_options\">GTK_FILL</property></pa"
  "cking></child><child><object class=\"GtkLabel\" id=\"power-label\"><pro"
  "perty name=\"visible\">True</property><property name=\"xalign\">0</prop"
  "erty><property name=\"label\" translatable=\"yes\">When power button is"
  " pressed:</property></object><packing><property name=\"top_attach\">1</"
  "property><property name=\"bottom_attach\">2</property><property name=\""
  "x_options\">GTK_FILL</property></packing></child><child><object class=\""
  "GtkComboBox\" id=\"tray-combox\"><property name=\"visible\">True</prope"
  "rty><signal name=\"changed\" handler=\"set_show_tray_icon_cb\"/><child>"
  "<object class=\"GtkCellRendererText\" id=\"renderer2\"/><attributes><at"
  "tribute name=\"text\">0</attribute></attributes></child></object><packi"
  "ng><property name=\"left_attach\">1</property><property name=\"right_at"
  "tach\">2</property><property name=\"y_padding\">3</property></packing><"
  "/child><child><object class=\"GtkLabel\" id=\"label24\"><property name="
  "\"visible\">True</property><property name=\"xalign\">0</property><prope"
  "rty name=\"label\" translatable=\"yes\">System tray icon: </property></"
  "object><packing><property name=\"x_options\">GTK_FILL</property></packi"
  "ng></child></object><packing><property name=\"expand\">False</property>"
  "<property name=\"padding\">3</property><property name=\"position\">0</p"
  "roperty></packing></child><child><object class=\"GtkCheckButton\" id=\""
  "enable-dpms\"><property name=\"label\" translatable=\"yes\">Monitor pow"
  "er management control</property><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">True</property><property name=\"receive"
  "s_default\">False</property><property name=\"draw_indicator\">True</pro"
  "perty><signal name=\"toggled\" handler=\"dpms_toggled_cb\"/></object><p"
  "acking><property name=\"expand\">False</property><property name=\"paddi"
  "ng\">3</property><property name=\"position\">1</property></packing></ch"
  "ild><child><object class=\"GtkCheckButton\" id=\"notification\"><proper"
  "ty name=\"label\" translatable=\"yes\">Show notifications to notify abo"
  "ut the battery state</property><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">True</property><property name=\"receives"
  "_default\">False</property><property name=\"draw_indicator\">True</prop"
  "erty><signal name=\"toggled\" handler=\"notify_toggled_cb\"/></object><"
  "packing><property name=\"expand\">False</property><property name=\"padd"
  "ing\">3</property><property name=\"position\">2</property></packing></c"
  "hild></object></child></object></child><child type=\"label\"><object cl"
  "ass=\"GtkLabel\" id=\"label26\"><property name=\"visible\">True</proper"
  "ty><property name=\"label\" translatable=\"yes\">&lt;b&gt;General Optio"
  "ns&lt;/b&gt;</property><property name=\"use_markup\">True</property></o"
  "bject></child></object></child></object></child><child type=\"tab\"><pl"
  "aceholder/></child><child><object class=\"GtkNotebook\" id=\"on-ac-note"
  "book\"><property name=\"visible\">True</property><property name=\"can_f"
  "ocus\">True</property><child><object class=\"GtkVBox\" id=\"vbox18\"><p"
  "roperty name=\"visible\">True</property><property name=\"border_width\""
  ">12</property><property name=\"orientation\">vertical</property><proper"
  "ty name=\"spacing\">6</property><child><object class=\"GtkFrame\" id=\""
  "on-ac-actions-frame\"><property name=\"visible\">True</property><proper"
  "ty name=\"label_xalign\">0</property><property name=\"shadow_type\">non"
  "e</property><child><object class=\"GtkAlignment\" id=\"alignment9\"><pr"
  "operty name=\"visible\">True</property><property name=\"top_padding\">6"
  "</property><property name=\"left_padding\">12</property><property name="
  "\"right_padding\">6</property><child><object class=\"GtkVBox\" id=\"vbo"
  "x19\"><property name=\"visible\">True</property><property name=\"orient"
  "ation\">vertical</property><property name=\"spacing\">6</property><chil"
  "d><object class=\"GtkHBox\" id=\"hbox17\"><property name=\"visible\">Tr"
  "ue</property><property name=\"spacing\">12</property><child><object cla"
  "ss=\"GtkLabel\" id=\"on-ac-lid-label\"><property name=\"visible\">True<"
  "/property><property name=\"label\" translatable=\"yes\">When laptop lid"
  " is closed:</property></object><packing><property name=\"expand\">False"
  "</property><property name=\"position\">0</property></packing></child><c"
  "hild><object class=\"GtkComboBox\" id=\"on-ac-lid\"><property name=\"vi"
  "sible\">True</property><property name=\"model\">model5</property><signa"
  "l name=\"changed\" handler=\"on_ac_lid_changed_cb\"/><child><object cla"
  "ss=\"GtkCellRendererText\" id=\"renderer5\"/><attributes><attribute nam"
  "e=\"text\">0</attribute></attributes></child></object><packing><propert"
  "y name=\"position\">1</property></packing></child></object><packing><pr"
  "operty name=\"expand\">False</property><property name=\"position\">0</p"
  "roperty></packing></child><child><object class=\"GtkVBox\" id=\"vbox1\""
  "><property name=\"visible\">True</property><property name=\"orientation"
  "\">vertical</property><property name=\"spacing\">4</property><child><ob"
  "ject class=\"GtkLabel\" id=\"label9\"><property name=\"visible\">True</"
  "property><property name=\"xalign\">0</property><property name=\"label\""
  " translatable=\"yes\">Put the computer to sleep when inactive for:</pro"
  "perty></object><packing><property name=\"position\">0</property></packi"
  "ng></child><child><object class=\"GtkHScale\" id=\"inactivity-on-ac\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">T"
  "rue</property><property name=\"update_policy\">discontinuous</property>"
  "<property name=\"adjustment\">adjustment1</property><property name=\"di"
  "gits\">0</property><property name=\"value_pos\">bottom</property><signa"
  "l name=\"value_changed\" handler=\"inactivity_on_ac_value_changed_cb\"/"
  "><signal name=\"format_value\" handler=\"format_inactivity_value_cb\"/>"
  "</object><packing><property name=\"position\">1</property></packing></c"
  "hild></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"position\">1</property></packing></child><child><object clas"
  "s=\"GtkCheckButton\" id=\"spin-down-on-ac\"><property name=\"label\" tr"
  "anslatable=\"yes\">Spin down hard disks</property><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">True</property><prope"
  "rty name=\"receives_default\">False</property><property name=\"draw_ind"
  "icator\">True</property><signal name=\"toggled\" handler=\"on_ac_spin_c"
  "hanged\"/></object><packing><property name=\"position\">2</property></p"
  "acking></child></object></child></object></child><child type=\"label\">"
  "<object class=\"GtkLabel\" id=\"sleep\"><property name=\"visible\">True"
  "</property><property name=\"label\" translatable=\"yes\">&lt;b&gt;Actio"
  "ns&lt;/b&gt;</property><property name=\"use_markup\">True</property></o"
  "bject></child></object><packing><property name=\"expand\">False</proper"
  "ty><property name=\"position\">0</property></packing></child></object><"
  "/child><child type=\"tab\"><object class=\"GtkLabel\" id=\"Actions-on-a"
  "c\"><property name=\"visible\">True</property><property name=\"label\" "
  "translatable=\"yes\">Actions</property></object><packing><property name"
  "=\"tab_fill\">False</property></packing></child><child><object class=\""
  "GtkVBox\" id=\"vbox6\"><property name=\"visible\">True</property><prope"
  "rty name=\"border_width\">12</property><property name=\"orientation\">v"
  "ertical</property><property name=\"spacing\">12</property><child><objec"
  "t class=\"GtkFrame\" id=\"dpms-on-ac-frame\"><property name=\"label_xal"
  "ign\">0</property><property name=\"shadow_type\">none</property><child>"
  "<object class=\"GtkAlignment\" id=\"alignment10\"><property name=\"visi"
  "ble\">True</property><property name=\"top_padding\">6</property><proper"
  "ty name=\"left_padding\">12</property><property name=\"right_padding\">"
  "6</property><child><object class=\"GtkVBox\" id=\"vbox20\"><property na"
  "me=\"visible\">True</property><property name=\"orientation\">vertical</"
  "property><property name=\"spacing\">6</property><child><object class=\""
  "GtkVBox\" id=\"vbox21\"><property name=\"visible\">True</property><prop"
  "erty name=\"orientation\">vertical</property><property name=\"spacing\""
  ">4</property><child><object class=\"GtkLabel\" id=\"label28\"><property"
  " name=\"visible\">True</property><property name=\"xalign\">0</property>"
  "<property name=\"label\" translatable=\"yes\">Put display to sleep when"
  " computer is inactive for:</property></object><packing><property name=\""
  "position\">0</property></packing></child><child><object class=\"GtkHSca"
  "le\" id=\"sleep-dpms-on-ac\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">True</property><property name=\"update_pol"
  "icy\">discontinuous</property><property name=\"adjustment\">adjustment2"
  "</property><property name=\"show_fill_level\">True</property><property "
  "name=\"digits\">0</property><property name=\"value_pos\">bottom</proper"
  "ty><signal name=\"value_changed\" handler=\"sleep_on_ac_value_changed_c"
  "b\"/><signal name=\"format_value\" handler=\"format_dpms_value_cb\"/></"
  "object><packing><property name=\"position\">1</property></packing></chi"
  "ld></object><packing><property name=\"expand\">False</property><propert"
  "y name=\"position\">0</property></packing></child><child><object class="
  "\"GtkVBox\" id=\"vbox22\"><property name=\"visible\">True</property><pr"
  "operty name=\"orientation\">vertical</property><property name=\"spacing"
  "\">4</property><child><object class=\"GtkLabel\" id=\"label29\"><proper"
  "ty name=\"visible\">True</property><property name=\"xalign\">0</propert"
  "y><property name=\"label\" translatable=\"yes\">Switch off display when"
  " computer is inactive for:</property></object><packing><property name=\""
  "position\">0</property></packing></child><child><object class=\"GtkHSca"
  "le\" id=\"off-dpms-on-ac\"><property name=\"visible\">True</property><p"
  "roperty name=\"can_focus\">True</property><property name=\"update_polic"
  "y\">discontinuous</property><property name=\"adjustment\">adjustment3</"
  "property><property name=\"show_fill_level\">True</property><property na"
  "me=\"digits\">0</property><property name=\"value_pos\">bottom</property"
  "><signal name=\"value_changed\" handler=\"off_on_ac_value_changed_cb\"/"
  "><signal name=\"format_value\" handler=\"format_dpms_value_cb\"/></obje"
  "ct><packing><property name=\"position\">1</property></packing></child><"
  "/object><packing><property name=\"expand\">False</property><property na"
  "me=\"position\">1</property></packing></child></object></child></object"
  "></child><child type=\"label\"><object class=\"GtkLabel\" id=\"label30\""
  "><property name=\"visible\">True</property><property name=\"label\" tra"
  "nslatable=\"yes\">&lt;b&gt;Monitor&lt;/b&gt;</property><property name=\""
  "use_markup\">True</property></object></child></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"position\">0</proper"
  "ty></packing></child><child><object class=\"GtkFrame\" id=\"on-ac-brg-f"
  "rame\"><property name=\"visible\">True</property><property name=\"label"
  "_xalign\">0</property><property name=\"shadow_type\">none</property><ch"
  "ild><object class=\"GtkAlignment\" id=\"alignment11\"><property name=\""
  "visible\">True</property><property name=\"top_padding\">6</property><pr"
  "operty name=\"left_padding\">12</property><property name=\"right_paddin"
  "g\">6</property><child><object class=\"GtkVBox\" id=\"vbox23\"><propert"
  "y name=\"visible\">True</property><property name=\"orientation\">vertic"
  "al</property><property name=\"spacing\">4</property><child><object clas"
  "s=\"GtkLabel\" id=\"label31\"><property name=\"visible\">True</property"
  "><property name=\"xalign\">0</property><property name=\"label\" transla"
  "table=\"yes\">Reduce screen brightness when computer is inactive for:</"
  "property></object><packing><property name=\"expand\">False</property><p"
  "roperty name=\"position\">0</property></packing></child><child><object "
  "class=\"GtkHScale\" id=\"brg-on-ac\"><property name=\"visible\">True</p"
  "roperty><property name=\"can_focus\">True</property><property name=\"up"
  "date_policy\">discontinuous</property><property name=\"adjustment\">adj"
  "ustment4</property><property name=\"digits\">0</property><property name"
  "=\"value_pos\">bottom</property><signal name=\"value_changed\" handler="
  "\"brightness_on_ac_value_changed_cb\"/><signal name=\"format_value\" ha"
  "ndler=\"format_brightness_value_cb\"/></object><packing><property name="
  "\"expand\">False</property><property name=\"position\">1</property></pa"
  "cking></child><child><object class=\"GtkHBox\" id=\"hbox1\"><property n"
  "ame=\"visible\">True</property><property name=\"spacing\">10</property>"
  "<child><object class=\"GtkLabel\" id=\"label1\"><property name=\"visibl"
  "e\">True</property><property name=\"label\" translatable=\"yes\">Level:"
  "</property></object><packing><property name=\"expand\">False</property>"
  "<property name=\"position\">0</property></packing></child><child><objec"
  "t class=\"GtkSpinButton\" id=\"brg-level-on-ac\"><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">True</property><proper"
  "ty name=\"invisible_char\">&#x25CF;</property><property name=\"adjustme"
  "nt\">adjustment10</property><property name=\"update_policy\">if-valid</"
  "property><signal name=\"value_changed\" handler=\"brightness_level_on_a"
  "c\"/><signal name=\"output\" handler=\"critical_spin_output_cb\"/></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">False</property><property name=\"position\">1</property></packin"
  "g></child></object><packing><property name=\"position\">2</property></p"
  "acking></child></object></child></object></child><child type=\"label\">"
  "<object class=\"GtkLabel\" id=\"label32\"><property name=\"visible\">Tr"
  "ue</property><property name=\"label\" translatable=\"yes\">&lt;b&gt;Bri"
  "ghtness&lt;/b&gt;</property><property name=\"use_markup\">True</propert"
  "y></object></child></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"position\">1</property></packing></child></obj"
  "ect><packing><property name=\"position\">1</property></packing></child>"
  "<child type=\"tab\"><object class=\"GtkLabel\" id=\"monitor-on-ac\"><pr"
  "operty name=\"visible\">True</property><property name=\"label\" transla"
  "table=\"yes\">Monitor</property></object><packing><property name=\"posi"
  "tion\">1</property><property name=\"tab_fill\">False</property></packin"
  "g></child><child><placeholder/></child><child type=\"tab\"><placeholder"
  "/></child></object><packing><property name=\"position\">1</property></p"
  "acking></child><child type=\"tab\"><placeholder/></child><child><object"
  " class=\"GtkNotebook\" id=\"on-battery-notebook\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">True</property><child"
  "><object class=\"GtkVBox\" id=\"vbox25\"><property name=\"visible\">Tru"
  "e</property><property name=\"border_width\">12</property><property name"
  "=\"orientation\">vertical</property><property name=\"spacing\">6</prope"
  "rty><child><object class=\"GtkFrame\" id=\"frame9\"><property name=\"vi"
  "sible\">True</property><property name=\"label_xalign\">0</property><pro"
  "perty name=\"shadow_type\">none</property><child><object class=\"GtkAli"
  "gnment\" id=\"alignment12\"><property name=\"visible\">True</property><"
  "property name=\"top_padding\">6</property><property name=\"left_padding"
  "\">12</property><property name=\"right_padding\">6</property><child><ob"
  "ject class=\"GtkVBox\" id=\"vbox26\"><property name=\"visible\">True</p"
  "roperty><property name=\"orientation\">vertical</property><property nam"
  "e=\"spacing\">6</property><child><object class=\"GtkTable\" id=\"table4"
  "\"><property name=\"visible\">True</property><property name=\"n_rows\">"
  "2</property><property name=\"n_columns\">2</property><property name=\"c"
  "olumn_spacing\">12</property><property name=\"row_spacing\">6</property"
  "><child><object class=\"GtkComboBox\" id=\"on-battery-lid\"><property n"
  "ame=\"visible\">True</property><property name=\"model\">model6</propert"
  "y><signal name=\"changed\" handler=\"on_battery_lid_changed_cb\"/><chil"
  "d><object class=\"GtkCellRendererText\" id=\"renderer6\"/><attributes><"
  "attribute name=\"text\">0</attribute></attributes></child></object><pac"
  "king><property name=\"left_attach\">1</property><property name=\"right_"
  "attach\">2</property><property name=\"top_attach\">1</property><propert"
  "y name=\"bottom_attach\">2</property></packing></child><child><object c"
  "lass=\"GtkLabel\" id=\"label34\"><property name=\"visible\">True</prope"
  "rty><property name=\"xalign\">0</property><property name=\"ypad\">4</pr"
  "operty><property name=\"label\" translatable=\"yes\">When battery power"
  " is critical:</property></object><packing><property name=\"x_options\">"
  "GTK_FILL</property></packing></child><child><object class=\"GtkLabel\" "
  "id=\"on-battery-lid-label\"><property name=\"visible\">True</property><"
  "property name=\"xalign\">0</property><property name=\"label\" translata"
  "ble=\"yes\">When laptop lid is closed:</property></object><packing><pro"
  "perty name=\"top_attach\">1</property><property name=\"bottom_attach\">"
  "2</property><property name=\"x_options\">GTK_FILL</property></packing><"
  "/child><child><object class=\"GtkComboBox\" id=\"battery-critical-combo"
  "x\"><property name=\"visible\">True</property><property name=\"model\">"
  "model7</property><signal name=\"changed\" handler=\"battery_critical_ch"
  "anged_cb\"/><child><object class=\"GtkCellRendererText\" id=\"renderer7"
  "\"/><attributes><attribute name=\"text\">0</attribute></attributes></ch"
  "ild></object><packing><property name=\"left_attach\">1</property><prope"
  "rty name=\"right_attach\">2</property></packing></child></object><packi"
  "ng><property name=\"expand\">False</property><property name=\"position\""
  ">0</property></packing></child><child><object class=\"GtkVBox\" id=\"vb"
  "ox4\"><property name=\"visible\">True</property><property name=\"orient"
  "ation\">vertical</property><property name=\"spacing\">4</property><chil"
  "d><object class=\"GtkLabel\" id=\"label6\"><property name=\"visible\">T"
  "rue</property><property name=\"xalign\">0</property><property name=\"la"
  "bel\" translatable=\"yes\">Put the computer to sleep when inactive for:"
  "</property></object><packing><property name=\"position\">0</property></"
  "packing></child><child><object class=\"GtkHScale\" id=\"inactivity-on-b"
  "attery\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">True</property><property name=\"update_policy\">discontinuous<"
  "/property><property name=\"adjustment\">adjustment5</property><property"
  " name=\"digits\">0</property><property name=\"value_pos\">bottom</prope"
  "rty><signal name=\"value_changed\" handler=\"inactivity_on_battery_valu"
  "e_changed_cb\"/><signal name=\"format_value\" handler=\"format_inactivi"
  "ty_value_cb\"/></object><packing><property name=\"position\">1</propert"
  "y></packing></child></object><packing><property name=\"position\">1</pr"
  "operty></packing></child><child><object class=\"GtkCheckButton\" id=\"p"
  "ower-save\"><property name=\"label\" translatable=\"yes\">Prefer power "
  "savings over performance</property><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">True</property><property name=\"rece"
  "ives_default\">False</property><property name=\"draw_indicator\">True</"
  "property><signal name=\"toggled\" handler=\"power_save_toggled_cb\"/></"
  "object><packing><property name=\"expand\">False</property><property nam"
  "e=\"position\">2</property></packing></child><child><object class=\"Gtk"
  "CheckButton\" id=\"spin-down-on-battery\"><property name=\"label\" tran"
  "slatable=\"yes\">Spin down hard disks</property><property name=\"visibl"
  "e\">True</property><property name=\"can_focus\">True</property><propert"
  "y name=\"receives_default\">False</property><property name=\"draw_indic"
  "ator\">True</property><signal name=\"toggled\" handler=\"on_battery_spi"
  "n_changed\"/></object><packing><property name=\"position\">3</property>"
  "</packing></child></object></child></object></child><child type=\"label"
  "\"><object class=\"GtkLabel\" id=\"label35\"><property name=\"visible\""
  ">True</property><property name=\"label\" translatable=\"yes\">&lt;b&gt;"
  "Actions&lt;/b&gt;</property><property name=\"use_markup\">True</propert"
  "y></object></child></object><packing><property name=\"expand\">False</p"
  "roperty><property name=\"position\">0</property></packing></child></obj"
  "ect></child><child type=\"tab\"><object class=\"GtkLabel\" id=\"actions"
  "-on-battery\"><property name=\"visible\">True</property><property name="
  "\"label\" translatable=\"yes\">Actions</property></object><packing><pro"
  "perty name=\"tab_fill\">False</property></packing></child><child><objec"
  "t class=\"GtkVBox\" id=\"vbox7\"><property name=\"visible\">True</prope"
  "rty><property name=\"border_width\">12</property><property name=\"orien"
  "tation\">vertical</property><property name=\"spacing\">12</property><ch"
  "ild><object class=\"GtkFrame\" id=\"dpms-on-battery-frame\"><property n"
  "ame=\"label_xalign\">0</property><property name=\"shadow_type\">none</p"
  "roperty><child><object class=\"GtkAlignment\" id=\"alignment13\"><prope"
  "rty name=\"visible\">True</property><property name=\"top_padding\">6</p"
  "roperty><property name=\"left_padding\">12</property><property name=\"r"
  "ight_padding\">6</property><child><object class=\"GtkVBox\" id=\"vbox27"
  "\"><property name=\"visible\">True</property><property name=\"orientati"
  "on\">vertical</property><property name=\"spacing\">6</property><child><"
  "object class=\"GtkVBox\" id=\"vbox28\"><property name=\"visible\">True<"
  "/property><property name=\"orientation\">vertical</property><property n"
  "ame=\"spacing\">4</property><child><object class=\"GtkLabel\" id=\"labe"
  "l36\"><property name=\"visible\">True</property><property name=\"xalign"
  "\">0</property><property name=\"label\" translatable=\"yes\">Put displa"
  "y to sleep when computer is inactive for:</property></object><packing><"
  "property name=\"expand\">False</property><property name=\"position\">0<"
  "/property></packing></child><child><object class=\"GtkHScale\" id=\"sle"
  "ep-dpms-on-battery\"><property name=\"visible\">True</property><propert"
  "y name=\"can_focus\">True</property><property name=\"update_policy\">di"
  "scontinuous</property><property name=\"adjustment\">adjustment6</proper"
  "ty><property name=\"show_fill_level\">True</property><property name=\"d"
  "igits\">0</property><property name=\"value_pos\">bottom</property><sign"
  "al name=\"value_changed\" handler=\"sleep_on_battery_value_changed_cb\""
  "/><signal name=\"format_value\" handler=\"format_dpms_value_cb\"/></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "position\">1</property></packing></child></object><packing><property na"
  "me=\"expand\">False</property><property name=\"position\">0</property><"
  "/packing></child><child><object class=\"GtkVBox\" id=\"vbox29\"><proper"
  "ty name=\"visible\">True</property><property name=\"orientation\">verti"
  "cal</property><property name=\"spacing\">4</property><child><object cla"
  "ss=\"GtkLabel\" id=\"label37\"><property name=\"visible\">True</propert"
  "y><property name=\"xalign\">0</property><property name=\"label\" transl"
  "atable=\"yes\">Switch off display when computer is inactive for:</prope"
  "rty></object><packing><property name=\"expand\">False</property><proper"
  "ty name=\"position\">0</property></packing></child><child><object class"
  "=\"GtkHScale\" id=\"off-dpms-on-battery\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">True</property><property name"
  "=\"update_policy\">discontinuous</property><property name=\"adjustment\""
  ">adjustment7</property><property name=\"show_fill_level\">True</propert"
  "y><property name=\"digits\">0</property><property name=\"value_pos\">bo"
  "ttom</property><signal name=\"value_changed\" handler=\"off_on_battery_"
  "value_changed_cb\"/><signal name=\"format_value\" handler=\"format_dpms"
  "_value_cb\"/></object><packing><property name=\"expand\">False</propert"
  "y><property name=\"position\">1</property></packing></child></object><p"
  "acking><property name=\"expand\">False</property><property name=\"posit"
  "ion\">1</property></packing></child></object></child></object></child><"
  "child type=\"label\"><object class=\"GtkLabel\" id=\"label38\"><propert"
  "y name=\"visible\">True</property><property name=\"label\" translatable"
  "=\"yes\">&lt;b&gt;Monitor&lt;/b&gt;</property><property name=\"use_mark"
  "up\">True</property></object></child></object><packing><property name=\""
  "expand\">False</property><property name=\"position\">0</property></pack"
  "ing></child><child><object class=\"GtkFrame\" id=\"on-battery-brg-frame"
  "\"><property name=\"visible\">True</property><property name=\"label_xal"
  "ign\">0</property><property name=\"shadow_type\">none</property><child>"
  "<object class=\"GtkAlignment\" id=\"alignment14\"><property name=\"visi"
  "ble\">True</property><property name=\"top_padding\">6</property><proper"
  "ty name=\"left_padding\">12</property><property name=\"right_padding\">"
  "6</property><child><object class=\"GtkVBox\" id=\"vbox30\"><property na"
  "me=\"visible\">True</property><property name=\"orientation\">vertical</"
  "property><property name=\"spacing\">4</property><child><object class=\""
  "GtkLabel\" id=\"label39\"><property name=\"visible\">True</property><pr"
  "operty name=\"xalign\">0</property><property name=\"label\" translatabl"
  "e=\"yes\">Reduce screen brightness when computer is inactive for:</prop"
  "erty></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"position\">0</property></packing></child><child><object clas"
  "s=\"GtkHScale\" id=\"brg-on-battery\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">True</property><property name=\"u"
  "pdate_policy\">discontinuous</property><property name=\"adjustment\">ad"
  "justment8</property><property name=\"digits\">0</property><property nam"
  "e=\"value_pos\">bottom</property><signal name=\"value_changed\" handler"
  "=\"brightness_on_battery_value_changed_cb\"/><signal name=\"format_valu"
  "e\" handler=\"format_brightness_value_cb\"/></object><packing><property"
  " name=\"expand\">False</property><property name=\"position\">1</propert"
  "y></packing></child><child><object class=\"GtkHBox\" id=\"hbox2\"><prop"
  "erty name=\"visible\">True</property><property name=\"spacing\">10</pro"
  "perty><child><object class=\"GtkLabel\" id=\"label2\"><property name=\""
  "visible\">True</property><property name=\"xalign\">0.46000000834465027<"
  "/property><property name=\"label\" translatable=\"yes\">Level:</propert"
  "y></object><packing><property name=\"expand\">False</property><property"
  " name=\"position\">0</property></packing></child><child><object class=\""
  "GtkSpinButton\" id=\"brg-level-on-battery\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">True</property><property na"
  "me=\"invisible_char\">&#x25CF;</property><property name=\"adjustment\">"
  "adjustment11</property><signal name=\"value_changed\" handler=\"brightn"
  "ess_level_on_battery\"/><signal name=\"output\" handler=\"critical_spin"
  "_output_cb\"/></object><packing><property name=\"expand\">False</proper"
  "ty><property name=\"position\">1</property></packing></child></object><"
  "packing><property name=\"position\">2</property></packing></child></obj"
  "ect></child></object></child><child type=\"label\"><object class=\"GtkL"
  "abel\" id=\"label40\"><property name=\"visible\">True</property><proper"
  "ty name=\"label\" translatable=\"yes\">&lt;b&gt;Brightness&lt;/b&gt;</p"
  "roperty><property name=\"use_markup\">True</property></object></child><"
  "/object><packing><property name=\"expand\">False</property><property na"
  "me=\"position\">1</property></packing></child></object><packing><proper"
  "ty name=\"position\">1</property></packing></child><child type=\"tab\">"
  "<object class=\"GtkLabel\" id=\"monitor-on-battery\"><property name=\"v"
  "isible\">True</property><property name=\"label\" translatable=\"yes\">M"
  "onitor</property></object><packing><property name=\"position\">1</prope"
  "rty><property name=\"tab_fill\">False</property></packing></child><chil"
  "d><placeholder/></child><child type=\"tab\"><placeholder/></child></obj"
  "ect><packing><property name=\"position\">2</property></packing></child>"
  "<child type=\"tab\"><placeholder/></child><child><object class=\"GtkVie"
  "wport\" id=\"advanced\"><property name=\"visible\">True</property><chil"
  "d><object class=\"GtkFrame\" id=\"frame1\"><property name=\"visible\">T"
  "rue</property><property name=\"border_width\">12</property><property na"
  "me=\"label_xalign\">0</property><property name=\"shadow_type\">none</pr"
  "operty><child><object class=\"GtkAlignment\" id=\"alignment1\"><propert"
  "y name=\"visible\">True</property><property name=\"left_padding\">12</p"
  "roperty><property name=\"right_padding\">6</property><child><object cla"
  "ss=\"GtkVBox\" id=\"vbox2\"><property name=\"visible\">True</property><"
  "property name=\"orientation\">vertical</property><property name=\"spaci"
  "ng\">6</property><child><object class=\"GtkTable\" id=\"table1\"><prope"
  "rty name=\"visible\">True</property><property name=\"n_rows\">3</proper"
  "ty><property name=\"n_columns\">2</property><property name=\"column_spa"
  "cing\">12</property><property name=\"row_spacing\">6</property><child><"
  "object class=\"GtkVBox\" id=\"vbox5\"><property name=\"visible\">True</"
  "property><property name=\"orientation\">vertical</property><child><obje"
  "ct class=\"GtkRadioButton\" id=\"inactivity-suspend\"><property name=\""
  "label\" translatable=\"yes\">Suspend</property><property name=\"visible"
  "\">True</property><property name=\"can_focus\">True</property><property"
  " name=\"receives_default\">False</property><property name=\"active\">Tr"
  "ue</property><property name=\"draw_indicator\">True</property><signal n"
  "ame=\"toggled\" handler=\"set_suspend_inactivity\"/></object><packing><"
  "property name=\"position\">0</property></packing></child><child><object"
  " class=\"GtkRadioButton\" id=\"inactivity-hibernate\"><property name=\""
  "label\" translatable=\"yes\">Hibernate</property><property name=\"visib"
  "le\">True</property><property name=\"can_focus\">True</property><proper"
  "ty name=\"receives_default\">False</property><property name=\"draw_indi"
  "cator\">True</property><property name=\"group\">inactivity-suspend</pro"
  "perty><signal name=\"toggled\" handler=\"set_hibernate_inactivity\"/></"
  "object><packing><property name=\"position\">1</property></packing></chi"
  "ld></object><packing><property name=\"left_attach\">1</property><proper"
  "ty name=\"right_attach\">2</property><property name=\"x_options\">GTK_F"
  "ILL</property></packing></child><child><object class=\"GtkVBox\" id=\"v"
  "box3\"><property name=\"visible\">True</property><property name=\"orien"
  "tation\">vertical</property><child><object class=\"GtkRadioButton\" id="
  "\"sleep-dpms-mode\"><property name=\"label\" translatable=\"yes\">Stand"
  "by</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">False</p"
  "roperty><property name=\"active\">True</property><property name=\"draw_"
  "indicator\">True</property><signal name=\"toggled\" handler=\"set_dpms_"
  "standby_mode\"/></object><packing><property name=\"position\">0</proper"
  "ty></packing></child><child><object class=\"GtkRadioButton\" id=\"suspe"
  "nd-dpms-mode\"><property name=\"label\" translatable=\"yes\">Suspend</p"
  "roperty><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"receives_default\">False</prope"
  "rty><property name=\"draw_indicator\">True</property><property name=\"g"
  "roup\">sleep-dpms-mode</property><signal name=\"toggled\" handler=\"set"
  "_dpms_suspend_mode\"/></object><packing><property name=\"position\">1</"
  "property></packing></child></object><packing><property name=\"left_atta"
  "ch\">1</property><property name=\"right_attach\">2</property><property "
  "name=\"top_attach\">1</property><property name=\"bottom_attach\">2</pro"
  "perty><property name=\"x_options\">GTK_FILL</property></packing></child"
  "><child><object class=\"GtkLabel\" id=\"label7\"><property name=\"visib"
  "le\">True</property><property name=\"xalign\">0</property><property nam"
  "e=\"label\" translatable=\"yes\">Set computer inactivity sleep mode:</p"
  "roperty></object><packing><property name=\"x_options\">GTK_FILL</proper"
  "ty></packing></child><child><object class=\"GtkLabel\" id=\"critical-le"
  "vel-label\"><property name=\"visible\">True</property><property name=\""
  "xalign\">0</property><property name=\"label\" translatable=\"yes\">Cons"
  "ider the computer on low power at:</property></object><packing><propert"
  "y name=\"top_attach\">2</property><property name=\"bottom_attach\">3</p"
  "roperty><property name=\"x_options\">GTK_FILL</property></packing></chi"
  "ld><child><object class=\"GtkSpinButton\" id=\"critical-spin\"><propert"
  "y name=\"visible\">True</property><property name=\"can_focus\">True</pr"
  "operty><property name=\"invisible_char\">&#x25CF;</property><property n"
  "ame=\"shadow_type\">none</property><property name=\"adjustment\">adjust"
  "ment9</property><signal name=\"value_changed\" handler=\"critical_level"
  "_value_changed_cb\"/><signal name=\"output\" handler=\"critical_spin_ou"
  "tput_cb\"/></object><packing><property name=\"left_attach\">1</property"
  "><property name=\"right_attach\">2</property><property name=\"top_attac"
  "h\">2</property><property name=\"bottom_attach\">3</property><property "
  "name=\"x_options\">GTK_FILL</property></packing></child><child><object "
  "class=\"GtkLabel\" id=\"dpms-mode-label\"><property name=\"visible\">Tr"
  "ue</property><property name=\"xalign\">0</property><property name=\"lab"
  "el\" translatable=\"yes\">Set monitor sleep mode:</property></object><p"
  "acking><property name=\"top_attach\">1</property><property name=\"botto"
  "m_attach\">2</property><property name=\"x_options\">GTK_FILL</property>"
  "</packing></child></object><packing><property name=\"expand\">False</pr"
  "operty><property name=\"position\">0</property></packing></child><child"
  "><object class=\"GtkCheckButton\" id=\"lock-screen\"><property name=\"l"
  "abel\" translatable=\"yes\">Lock screen when going for suspend/hibernat"
  "e</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">False</p"
  "roperty><property name=\"draw_indicator\">True</property><signal name=\""
  "toggled\" handler=\"lock_screen_toggled_cb\"/></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"position\">1</prope"
  "rty></packing></child></object></child></object></child><child type=\"l"
  "abel\"><object class=\"GtkLabel\" id=\"label5\"><property name=\"visibl"
  "e\">True</property><property name=\"label\" translatable=\"yes\">&lt;b&"
  "gt;Advanced Options&lt;/b&gt;</property><property name=\"use_markup\">T"
  "rue</property></object></child></object></child></object><packing><prop"
  "erty name=\"position\">3</property></packing></child><child type=\"tab\""
  "><placeholder/></child></object><packing><property name=\"position\">1<"
  "/property></packing></child></object><packing><property name=\"position"
  "\">0</property></packing></child></object><packing><property name=\"pos"
  "ition\">1</property></packing></child><child internal-child=\"action_ar"
  "ea\"><object class=\"GtkHButtonBox\" id=\"dialog-action_area\"><propert"
  "y name=\"visible\">True</property><child><object class=\"GtkButton\" id"
  "=\"help-button\"><property name=\"label\">gtk-help</property><property "
  "name=\"visible\">True</property><property name=\"can_focus\">True</prop"
  "erty><property name=\"receives_default\">True</property><property name="
  "\"use_stock\">True</property></object><packing><property name=\"expand\""
  ">False</property><property name=\"fill\">False</property><property name"
  "=\"position\">0</property></packing></child><child><object class=\"GtkB"
  "utton\" id=\"close-button\"><property name=\"label\">gtk-close</propert"
  "y><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"receives_default\">True</property><pro"
  "perty name=\"use_stock\">True</property></object><packing><property nam"
  "e=\"expand\">False</property><property name=\"fill\">False</property><p"
  "roperty name=\"position\">1</property></packing></child></object><packi"
  "ng><property name=\"expand\">False</property><property name=\"pack_type"
  "\">end</property><property name=\"position\">0</property></packing></ch"
  "ild></object></child><action-widgets><action-widget response=\"0\">help"
  "-button</action-widget><action-widget response=\"0\">close-button</acti"
  "on-widget></action-widgets></object><object class=\"GtkAdjustment\" id="
  "\"adjustment10\"><property name=\"value\">80</property><property name=\""
  "lower\">1</property><property name=\"upper\">100</property><property na"
  "me=\"step_increment\">1</property></object><object class=\"GtkAdjustmen"
  "t\" id=\"adjustment11\"><property name=\"value\">20</property><property"
  " name=\"lower\">1</property><property name=\"upper\">100</property><pro"
  "perty name=\"step_increment\">1</property></object></interface>"
};

static const unsigned xfpm_settings_ui_length = 41982u;

