


#include <xfpm-enum-types.h>
#include "xfpm-enum-glib.h"


/* enumerations from "./xfpm-enum-glib.h" */
GType
xfpm_device_type_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ XFPM_DEVICE_TYPE_UNKNOWN, "XFPM_DEVICE_TYPE_UNKNOWN", "unknown" },
	{ XFPM_DEVICE_TYPE_LINE_POWER, "XFPM_DEVICE_TYPE_LINE_POWER", "line-power" },
	{ XFPM_DEVICE_TYPE_BATTERY, "XFPM_DEVICE_TYPE_BATTERY", "battery" },
	{ XFPM_DEVICE_TYPE_UPS, "XFPM_DEVICE_TYPE_UPS", "ups" },
	{ XFPM_DEVICE_TYPE_MONITOR, "XFPM_DEVICE_TYPE_MONITOR", "monitor" },
	{ XFPM_DEVICE_TYPE_MOUSE, "XFPM_DEVICE_TYPE_MOUSE", "mouse" },
	{ XFPM_DEVICE_TYPE_KBD, "XFPM_DEVICE_TYPE_KBD", "kbd" },
	{ XFPM_DEVICE_TYPE_PDA, "XFPM_DEVICE_TYPE_PDA", "pda" },
	{ XFPM_DEVICE_TYPE_PHONE, "XFPM_DEVICE_TYPE_PHONE", "phone" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmDeviceType", values);
  }
	return type;
}
GType
xfpm_device_state_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ XFPM_DEVICE_STATE_UNKNOWN, "XFPM_DEVICE_STATE_UNKNOWN", "unknown" },
	{ XFPM_DEVICE_STATE_CHARGING, "XFPM_DEVICE_STATE_CHARGING", "charging" },
	{ XFPM_DEVICE_STATE_DISCHARGING, "XFPM_DEVICE_STATE_DISCHARGING", "discharging" },
	{ XFPM_DEVICE_STATE_EMPTY, "XFPM_DEVICE_STATE_EMPTY", "empty" },
	{ XFPM_DEVICE_STATE_FULLY_CHARGED, "XFPM_DEVICE_STATE_FULLY_CHARGED", "fully-charged" },
	{ XFPM_DEVICE_STATE_PENDING_CHARGING, "XFPM_DEVICE_STATE_PENDING_CHARGING", "pending-charging" },
	{ XFPM_DEVICE_STATE_PENDING_DISCHARGING, "XFPM_DEVICE_STATE_PENDING_DISCHARGING", "pending-discharging" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmDeviceState", values);
  }
	return type;
}
GType
xfpm_battery_charge_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ XFPM_BATTERY_CHARGE_UNKNOWN, "XFPM_BATTERY_CHARGE_UNKNOWN", "unknown" },
	{ XFPM_BATTERY_CHARGE_CRITICAL, "XFPM_BATTERY_CHARGE_CRITICAL", "critical" },
	{ XFPM_BATTERY_CHARGE_LOW, "XFPM_BATTERY_CHARGE_LOW", "low" },
	{ XFPM_BATTERY_CHARGE_OK, "XFPM_BATTERY_CHARGE_OK", "ok" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmBatteryCharge", values);
  }
	return type;
}
GType
xfpm_shutdown_request_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ XFPM_DO_NOTHING, "XFPM_DO_NOTHING", "do-nothing" },
	{ XFPM_DO_SUSPEND, "XFPM_DO_SUSPEND", "do-suspend" },
	{ XFPM_DO_HIBERNATE, "XFPM_DO_HIBERNATE", "do-hibernate" },
	{ XFPM_ASK, "XFPM_ASK", "ask" },
	{ XFPM_DO_SHUTDOWN, "XFPM_DO_SHUTDOWN", "do-shutdown" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmShutdownRequest", values);
  }
	return type;
}
GType
xfpm_lid_trigger_action_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ LID_TRIGGER_NOTHING, "LID_TRIGGER_NOTHING", "nothing" },
	{ LID_TRIGGER_SUSPEND, "LID_TRIGGER_SUSPEND", "suspend" },
	{ LID_TRIGGER_HIBERNATE, "LID_TRIGGER_HIBERNATE", "hibernate" },
	{ LID_TRIGGER_LOCK_SCREEN, "LID_TRIGGER_LOCK_SCREEN", "lock-screen" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmLidTriggerAction", values);
  }
	return type;
}
GType
xfpm_button_key_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ BUTTON_UNKNOWN, "BUTTON_UNKNOWN", "button-unknown" },
	{ BUTTON_POWER_OFF, "BUTTON_POWER_OFF", "button-power-off" },
	{ BUTTON_HIBERNATE, "BUTTON_HIBERNATE", "button-hibernate" },
	{ BUTTON_SLEEP, "BUTTON_SLEEP", "button-sleep" },
	{ BUTTON_MON_BRIGHTNESS_UP, "BUTTON_MON_BRIGHTNESS_UP", "button-mon-brightness-up" },
	{ BUTTON_MON_BRIGHTNESS_DOWN, "BUTTON_MON_BRIGHTNESS_DOWN", "button-mon-brightness-down" },
	{ BUTTON_LID_CLOSED, "BUTTON_LID_CLOSED", "button-lid-closed" },
	{ BUTTON_BATTERY, "BUTTON_BATTERY", "button-battery" },
	{ NUMBER_OF_BUTTONS, "NUMBER_OF_BUTTONS", "number-of-buttons" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmButtonKey", values);
  }
	return type;
}
GType
xfpm_show_icon_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ SHOW_ICON_ALWAYS, "SHOW_ICON_ALWAYS", "show-icon-always" },
	{ SHOW_ICON_WHEN_BATTERY_PRESENT, "SHOW_ICON_WHEN_BATTERY_PRESENT", "show-icon-when-battery-present" },
	{ SHOW_ICON_WHEN_BATTERY_CHARGING_DISCHARGING, "SHOW_ICON_WHEN_BATTERY_CHARGING_DISCHARGING", "show-icon-when-battery-charging-discharging" },
	{ NEVER_SHOW_ICON, "NEVER_SHOW_ICON", "never-show-icon" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmShowIcon", values);
  }
	return type;
}
GType
xfpm_power_mode_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
	static const GEnumValue values[] = {
	{ XFPM_POWER_MODE_NORMAL, "XFPM_POWER_MODE_NORMAL", "normal" },
	{ XFPM_POWER_MODE_PRESENTATION, "XFPM_POWER_MODE_PRESENTATION", "presentation" },
	{ 0, NULL, NULL }
	};
	type = g_enum_register_static ("XfpmPowerMode", values);
  }
	return type;
}



