/* automatically generated from acme.ui */
#ifdef __SUNPRO_C
#pragma align 4 (acme_ui)
#endif
#ifdef __GNUC__
static const char acme_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char acme_ui[] =
#endif
{
  "<?xml version=\"1.0\"?><interface><object class=\"GtkWindow\" id=\"dial"
  "og\"><child><object class=\"GtkFrame\" id=\"acme_frame\"><property name"
  "=\"visible\">True</property><property name=\"label_xalign\">0</property"
  "><property name=\"shadow_type\">out</property><child><object class=\"Gt"
  "kVBox\" id=\"acme_vbox\"><property name=\"visible\">True</property><pro"
  "perty name=\"border_width\">12</property><property name=\"spacing\">6</"
  "property><child><object class=\"GtkImage\" id=\"acme_image\"><property "
  "name=\"visible\">True</property><property name=\"icon_name\">audio-volu"
  "me-high</property><property name=\"icon-size\">6</property></object><pa"
  "cking><property name=\"position\">0</property></packing></child><child>"
  "<object class=\"GtkProgressBar\" id=\"acme_volume_progressbar\"><proper"
  "ty name=\"visible\">True</property></object><packing><property name=\"e"
  "xpand\">False</property><property name=\"fill\">False</property><proper"
  "ty name=\"position\">1</property></packing></child></object></child></o"
  "bject></child></object></interface>"
};

static const unsigned acme_ui_length = 979u;

